@@ -29,6 +29,7 @@ from shapely.geometry import (
 )
 
 XY = Tuple[float, float]
+XYZ = Tuple[float, float, float]
 
 
 class AntimeridianWarning(UserWarning):
@@ -55,12 +56,12 @@ class FixWindingWarning(AntimeridianWarning):
 
 
 class GeoInterface(Protocol):
-    """A simple protocol for things that have a ``__geo_interface__`` method.
+    """A simple protocol for things that have a `__geo_interface__` method.
 
-    The ``__geo_interface__`` protocol is described `here
-    <https://gist.github.com/sgillies/2217756>`_, and is used within `shapely
-    <https://shapely.readthedocs.io/en/stable/manual.html>`_ to
-    extract geometries from objects.
+    The `__geo_interface__` protocol is described
+    [here](https://gist.github.com/sgillies/2217756>), and is used within
+    [shapely](https://shapely.readthedocs.io/en/stable/manual.html) to extract
+    geometries from objects.
     """
 
     @property
@@ -73,13 +74,14 @@ def fix_geojson(
     force_north_pole: bool = False,
     force_south_pole: bool = False,
     fix_winding: bool = True,
+    great_circle: bool = True,
 ) -> Dict[str, Any]:
     """Fixes a GeoJSON object that crosses the antimeridian.
 
     If the object does not cross the antimeridian, it is returned unchanged.
 
-    See :py:func:`fix_polygon` for a description of the ``force_north_pole``,
-    ``force_south_pole``, and ``fix_winding`` arguments.
+    See [antimeridian.fix_polygon][] for a description of the `force_north_pole`
+    `force_south_pole` and `fix_winding` arguments.
 
     Args:
         geojson: A GeoJSON object as a dictionary
@@ -89,6 +91,8 @@ def fix_geojson(
             joined segments to enclose the south pole.
         fix_winding: If the polygon is wound clockwise, reverse its
             coordinates before applying the algorithm.
+        great_circle: Compute meridian crossings on the sphere rather than
+            using 2D geometry.
 
     Return:
         The same GeoJSON with a fixed geometry or geometries
@@ -105,6 +109,7 @@ def fix_geojson(
             force_north_pole=force_north_pole,
             force_south_pole=force_south_pole,
             fix_winding=fix_winding,
+            great_circle=great_circle,
         )
         return geojson
     elif type_ == "FeatureCollection":
@@ -117,6 +122,7 @@ def fix_geojson(
                 force_north_pole=force_north_pole,
                 force_south_pole=force_south_pole,
                 fix_winding=fix_winding,
+                great_circle=great_circle,
             )
         geojson["features"] = features
         return geojson
@@ -126,10 +132,11 @@ def fix_geojson(
             force_north_pole=force_north_pole,
             force_south_pole=force_south_pole,
             fix_winding=fix_winding,
+            great_circle=great_circle,
         )
 
 
-def segment_geojson(geojson: Dict[str, Any]) -> MultiLineString:
+def segment_geojson(geojson: Dict[str, Any], great_circle: bool) -> MultiLineString:
     """Segments a GeoJSON object into a MultiLineString.
 
     If the object does not cross the antimeridian, its exterior and interior
@@ -137,6 +144,8 @@ def segment_geojson(geojson: Dict[str, Any]) -> MultiLineString:
 
     Args:
         geojson: A GeoJSON object as a dictionary
+        great_circle: Compute meridian crossings on the sphere rather than
+            using 2D geometry.
 
     Return:
         A MutliLineString of segments.
@@ -148,17 +157,17 @@ def segment_geojson(geojson: Dict[str, Any]) -> MultiLineString:
         geometry = geojson.get("geometry", None)
         if geometry is None:
             raise ValueError("no 'geometry' field found in GeoJSON Feature")
-        return MultiLineString(segment_shape(geometry))
+        return MultiLineString(segment_shape(geometry, great_circle))
     elif type_ == "FeatureCollection":
         features = geojson.get("features", None)
         if features is None:
             raise ValueError("no 'features' field found in GeoJSON FeatureCollection")
         segments = list()
         for feature in features:
-            segments.extend(segment_geojson(feature))
+            segments.extend(segment_geojson(feature, great_circle))
         return MultiLineString(segments)
     else:
-        return MultiLineString(segment_shape(geojson))
+        return MultiLineString(segment_shape(geojson, great_circle))
 
 
 def fix_shape(
@@ -167,22 +176,25 @@ def fix_shape(
     force_north_pole: bool = False,
     force_south_pole: bool = False,
     fix_winding: bool = True,
+    great_circle: bool = True,
 ) -> Dict[str, Any]:
     """Fixes a shape that crosses the antimeridian.
 
-    See :py:func:`fix_polygon` for a description of the ``force_north_pole``,
-    ``force_south_pole``, and ``fix_winding`` arguments.
+    See [antimeridian.fix_polygon][] for a description of the `force_north_pole`
+    `force_south_pole` and `fix_winding` arguments.
 
     Args:
         shape: A polygon, multi-polygon, line string, or multi-line string,
-            either as a dictionary or as a :py:class:`GeoInterface`. Uses
-            :py:func:`shapely.geometry.shape` under the hood.
+            either as a dictionary or as a [antimeridian.GeoInterface][]. Uses
+            [shapely.geometry.shape][] under the hood.
         force_north_pole: If the polygon crosses the antimeridian, force the
             joined segments to enclose the north pole.
         force_south_pole: If the polygon crosses the antimeridian, force the
             joined segments to enclose the south pole.
         fix_winding: If the polygon is wound clockwise, reverse its
             coordinates before applying the algorithm.
+        great_circle: Compute meridian crossings on the sphere rather than
+            using 2D geometry.
 
     Returns:
         The fixed shape as a dictionary
@@ -197,6 +209,7 @@ def fix_shape(
                     force_north_pole=force_north_pole,
                     force_south_pole=force_south_pole,
                     fix_winding=fix_winding,
+                    great_circle=great_circle,
                 )
             ),
         )
@@ -209,27 +222,34 @@ def fix_shape(
                     force_north_pole=force_north_pole,
                     force_south_pole=force_south_pole,
                     fix_winding=fix_winding,
+                    great_circle=great_circle,
                 )
             ),
         )
     elif geom.geom_type == "LineString":
-        return cast(Dict[str, Any], shapely.geometry.mapping(fix_line_string(geom)))
+        return cast(
+            Dict[str, Any],
+            shapely.geometry.mapping(fix_line_string(geom, great_circle)),
+        )
     elif geom.geom_type == "MultiLineString":
         return cast(
-            Dict[str, Any], shapely.geometry.mapping(fix_multi_line_string(geom))
+            Dict[str, Any],
+            shapely.geometry.mapping(fix_multi_line_string(geom, great_circle)),
         )
     else:
         raise ValueError(f"unsupported geom_type: {geom.geom_type}")
 
 
-def segment_shape(shape: Dict[str, Any] | GeoInterface) -> List[List[XY]]:
+def segment_shape(
+    shape: Dict[str, Any] | GeoInterface, great_circle: bool
+) -> List[List[XY]]:
     geom = shapely.geometry.shape(shape)
     if geom.geom_type == "Polygon":
-        return segment_polygon(geom)
+        return segment_polygon(geom, great_circle)
     elif geom.geom_type == "MultiPolygon":
         segments = list()
         for polygon in geom.geoms:
-            segments += segment_polygon(polygon)
+            segments += segment_polygon(polygon, great_circle)
         return segments
     else:
         raise ValueError(f"unsupported geom_type: {geom.geom_type}")
@@ -241,11 +261,12 @@ def fix_multi_polygon(
     force_north_pole: bool = False,
     force_south_pole: bool = False,
     fix_winding: bool = True,
+    great_circle: bool = True,
 ) -> MultiPolygon:
-    """Fixes a :py:class:`shapely.geometry.MultiPolygon`.
+    """Fixes a [shapely.MultiPolygon][].
 
-    See :py:func:`fix_polygon` for a description of the ``force_north_pole``,
-    ``force_south_pole``, and ``fix_winding`` arguments.
+    See [antimeridian.fix_polygon][] for a description of the `force_north_pole`
+    `force_south_pole` and `fix_winding` arguments.
 
     Args:
         multi_polygon: The multi-polygon
@@ -255,6 +276,8 @@ def fix_multi_polygon(
             joined segments to enclose the south pole.
         fix_winding: If the polygon is wound clockwise, reverse its
             coordinates before applying the algorithm.
+        great_circle: Compute meridian crossings on the sphere rather than
+            using 2D geometry.
 
     Returns:
         The fixed multi-polygon
@@ -266,6 +289,7 @@ def fix_multi_polygon(
             force_north_pole=force_north_pole,
             force_south_pole=force_south_pole,
             fix_winding=fix_winding,
+            great_circle=great_circle,
         )
     return MultiPolygon(polygons)
 
@@ -276,23 +300,24 @@ def fix_polygon(
     force_north_pole: bool = False,
     force_south_pole: bool = False,
     fix_winding: bool = True,
+    great_circle: bool = True,
 ) -> Union[Polygon, MultiPolygon]:
-    """Fixes a :py:class:`shapely.geometry.Polygon`.
+    """Fixes a [shapely.Polygon][].
 
     If the input polygon is wound clockwise, it will be fixed to be wound
-    counterclockwise _unless_ ``fix_winding`` is ``False``, in which case it
+    counterclockwise _unless_ `fix_winding` is `False` in which case it
     will be corrected by adding a counterclockwise polygon from (-180, -90) to
     (180, 90) as its exterior.
 
     In rare cases, the underlying algorithm might need a little help to fix the polygon.
     For example, a polygon that just barely crosses over a pole might have very
     few points at high latitudes, leading to ambiguous antimeridian crossing
-    points and invalid geometries. We provide two flags, ``force_north_pole``
-    and ``force_south_pole``, for those cases. Most users can ignore these
+    points and invalid geometries. We provide two flags, `force_north_pole`
+    and `force_south_pole` for those cases. Most users can ignore these
     flags.
 
-    If either ``force_north_pole`` or ``force_south_pole`` is ``True``,
-    ``fix_winding`` is set to ``False``.
+    If either `force_north_pole` or `force_south_pole` is `True`
+    `fix_winding` is set to `False`
 
     Args:
         polygon: The input polygon
@@ -302,6 +327,8 @@ def fix_polygon(
             joined segments to enclose the south pole.
         fix_winding: If the polygon is wound clockwise, reverse its
             coordinates before applying the algorithm.
+        great_circle: Compute meridian crossings on the sphere rather than
+            using 2D geometry.
 
     Returns:
         The fixed polygon, either as a single polygon or a multi-polygon (if it
@@ -314,6 +341,7 @@ def fix_polygon(
         force_north_pole=force_north_pole,
         force_south_pole=force_south_pole,
         fix_winding=fix_winding,
+        great_circle=great_circle,
     )
     if len(polygons) == 1:
         polygon = polygons[0]
@@ -328,35 +356,43 @@ def fix_polygon(
         return MultiPolygon(polygons)
 
 
-def fix_line_string(line_string: LineString) -> Union[LineString, MultiLineString]:
-    """Fixes a :py:class:`shapely.geometry.LineString`.
+def fix_line_string(
+    line_string: LineString, great_circle: bool
+) -> Union[LineString, MultiLineString]:
+    """Fixes a [shapely.LineString][].
 
     Args:
         line_string: The input line string
+        great_circle: Compute meridian crossings on the sphere rather than
+            using 2D geometry.
 
     Returns:
         The fixed line string, either as a single line string or a multi-line
         string (if it was split)
     """
-    segments = segment(list(line_string.coords))
+    segments = segment(list(line_string.coords), great_circle)
     if not segments:
         return line_string
     else:
         return MultiLineString(segments)
 
 
-def fix_multi_line_string(multi_line_string: MultiLineString) -> MultiLineString:
-    """Fixes a :py:class:`shapely.geometry.MultiLineString`.
+def fix_multi_line_string(
+    multi_line_string: MultiLineString, great_circle: bool
+) -> MultiLineString:
+    """Fixes a [shapely.MultiLineString][].
 
     Args:
         multi_line_string: The input multi line string
+        great_circle: Compute meridian crossings on the sphere rather than
+            using 2D geometry.
 
     Returns:
         The fixed multi line string
     """
     line_strings = list()
     for line_string in multi_line_string.geoms:
-        fixed = fix_line_string(line_string)
+        fixed = fix_line_string(line_string, great_circle)
         if isinstance(fixed, LineString):
             line_strings.append(fixed)
         else:
@@ -364,12 +400,12 @@ def fix_multi_line_string(multi_line_string: MultiLineString) -> MultiLineString
     return MultiLineString(line_strings)
 
 
-def segment_polygon(polygon: Polygon) -> List[List[XY]]:
-    segments = segment(list(polygon.exterior.coords))
+def segment_polygon(polygon: Polygon, great_circle: bool) -> List[List[XY]]:
+    segments = segment(list(polygon.exterior.coords), great_circle)
     if not segments:
         segments = [list(polygon.exterior.coords)]
     for interior in polygon.interiors:
-        interior_segments = segment(list(interior.coords))
+        interior_segments = segment(list(interior.coords), great_circle)
         if interior_segments:
             segments.extend(interior_segments)
         else:
@@ -383,9 +419,10 @@ def fix_polygon_to_list(
     force_north_pole: bool,
     force_south_pole: bool,
     fix_winding: bool,
+    great_circle: bool,
 ) -> List[Polygon]:
     exterior = normalize(list(polygon.exterior.coords))
-    segments = segment(exterior)
+    segments = segment(exterior, great_circle)
     if not segments:
         polygon = Polygon(shell=exterior, holes=polygon.interiors)
         if fix_winding and (
@@ -399,7 +436,7 @@ def fix_polygon_to_list(
     else:
         interiors = []
         for interior in polygon.interiors:
-            interior_segments = segment(list(interior.coords))
+            interior_segments = segment(list(interior.coords), great_circle)
             if interior_segments:
                 if fix_winding:
                     unwrapped_linearring = LinearRing(
@@ -407,7 +444,9 @@ def fix_polygon_to_list(
                     )
                     if shapely.is_ccw(unwrapped_linearring):
                         FixWindingWarning.warn()
-                        interior_segments = segment(list(reversed(interior.coords)))
+                        interior_segments = segment(
+                            list(reversed(interior.coords)), great_circle
+                        )
                 segments.extend(interior_segments)
             else:
                 interiors.append(interior)
@@ -465,18 +504,18 @@ def normalize(coords: List[XY]) -> List[XY]:
         return coords
 
 
-def segment(coords: List[XY]) -> List[List[XY]]:
+def segment(coords: List[XY], great_circle: bool) -> List[List[XY]]:
     segment = []
     segments = []
     for start, end in itertools.pairwise(coords):
         segment.append(start)
         if (end[0] - start[0] > 180) and (end[0] - start[0] != 360):  # left
-            latitude = crossing_latitude(start, end)
+            latitude = crossing_latitude(start, end, great_circle)
             segment.append((-180, latitude))
             segments.append(segment)
             segment = [(180, latitude)]
         elif (start[0] - end[0] > 180) and (start[0] - end[0] != 360):  # right
-            latitude = crossing_latitude(end, start)
+            latitude = crossing_latitude(end, start, great_circle)
             segment.append((180, latitude))
             segments.append(segment)
             segment = [(-180, latitude)]
@@ -492,11 +531,34 @@ def segment(coords: List[XY]) -> List[List[XY]]:
     return segments
 
 
-def crossing_latitude(start: XY, end: XY) -> float:
-    if abs(start[0]) == 180:
-        return start[1]
-    elif abs(end[0]) == 180:
-        return end[1]
+def spherical_degrees_to_cartesian(point: XY) -> XYZ:
+    lon, lat = numpy.deg2rad(point)
+    return (
+        float(numpy.cos(lon) * numpy.cos(lat)),
+        float(numpy.sin(lon) * numpy.cos(lat)),
+        float(numpy.sin(lat)),
+    )
+
+
+def crossing_latitude_great_circle(start: XY, end: XY) -> float:
+    # Compute a 3D vector representing the start and end points.
+    p1 = spherical_degrees_to_cartesian(start)
+    p2 = spherical_degrees_to_cartesian(end)
+    # The cross product of these vectors defines the plane passing through both points.
+    n1 = numpy.cross(p1, p2)
+    # The unit vector -Y defines the meridian plane.
+    n2 = numpy.array([0, -1, 0])
+    # The intersection of both planes is defined by their cross product, which we
+    # normalize to get its intersection with the unit sphere.
+    intersection = numpy.cross(n1, n2)
+    intersection /= numpy.linalg.norm(intersection)
+    # Convert back to spherical coordinates.
+    # We're only interested in the latitude, so the arcsin of the z-coordinate is
+    # sufficient.
+    return round(float(numpy.rad2deg(numpy.arcsin(intersection[2]))), 7)
+
+
+def crossing_latitude_flat(start: XY, end: XY) -> float:
     latitude_delta = end[1] - start[1]
     if end[0] > 0:
         return round(
@@ -512,6 +574,17 @@ def crossing_latitude(start: XY, end: XY) -> float:
         )
 
 
+def crossing_latitude(start: XY, end: XY, great_circle: bool) -> float:
+    if abs(start[0]) == 180:
+        return start[1]
+    elif abs(end[0]) == 180:
+        return end[1]
+
+    if great_circle:
+        return crossing_latitude_great_circle(start, end)
+    return crossing_latitude_flat(start, end)
+
+
 IndexAndLatitude = namedtuple("IndexAndLatitude", "index latitude")
 
 
@@ -668,14 +741,14 @@ def bbox(
 ) -> List[float]:
     """Calculates a GeoJSON-spec conforming bounding box for a shape.
 
-    Per `the GeoJSON spec
-    <https://datatracker.ietf.org/doc/html/rfc7946#section-5.2>`_, an
+    Per the [GeoJSON
+    spec](https://datatracker.ietf.org/doc/html/rfc7946#section-5.2), an
     antimeridian-spanning bounding box should have its larger longitude as its
     first bounding box coordinate.
 
     Args:
         shape: The polygon or multipolygon for which to calculate the bounding box.
-        force_over_antimeridan: Force the bounding box to be over the antimeridian.
+        force_over_antimeridian: Force the bounding box to be over the antimeridian.
 
     Returns:
         List[float]: The bounding box.
@@ -715,7 +788,7 @@ def bbox(
 def centroid(shape: Dict[str, Any] | GeoInterface) -> Point:
     """Calculates the centroid for a polygon or multipolygon.
 
-    Polygons are easy, we just use :py:func:`shapely.centroid`. For
+    Polygons are easy, we just use [shapely.centroid][]. For
     multi-polygons, the antimeridian is taken into account by calculating the
     centroid from an identical multi-polygon with coordinates in [0, 360).
 
